import confirm from 'ant-design-vue/es/modal/confirm'
import notification from 'ant-design-vue/es/notification'
import { getLodop } from '@/libs/LodopFuncs'
import { printLogSave } from '@/api/data'
import moment from 'moment'
// 打印页签,支持批量打印
export const JGPrintTag = function (html, width, height, data) {
  const LODOP = getLodop(document.getElementById('LODOP_OB'), document.getElementById('LODOP_EM'))
  if (!LODOP) {
    confirm({
      title: '提示?',
      content: h => <div>打印控件未安装，请先下载并安装。安装完成后，刷新页面即可打印。</div>,
      okText: '立即下载',
      okType: 'danger',
      cancelText: '暂不打印',
      onOk () {
        var agent = navigator.userAgent.toLowerCase();
        if (agent.indexOf("win32") >= 0 || agent.indexOf("wow32") >= 0) {
            window.open('http://www.lodop.net/demolist/CLodop_Setup_for_Win32NT.zip')
        }else if (agent.indexOf("win64") >= 0 || agent.indexOf("wow64") >= 0) {
            window.open('http://www.lodop.net/download/CLodop_Setup_for_Win64NT_4.155EN.zip')
        }
      }
    })
    return
  }
  LODOP.PRINT_INIT("")
  LODOP.SET_SHOW_MODE('HIDE_PAPER_BOARD', 1) //  隐藏底图上有模拟走纸板的条纹线
  LODOP.SET_PRINT_MODE('POS_BASEON_PAPER', true) // 可使输出以纸张边缘为基点
  LODOP.ADD_PRINT_HTML(0, 0, '100%', '100%', html)
  LODOP.ADD_PRINT_BARCODE('36%','65%',90,90,"QRCode",data.qrCodeContent)
  LODOP.SET_PRINT_STYLEA(0,"QRCodeVersion",5)
  // LODOP.SET_PRINT_STYLEA(0,"QRCodeErrorLevel",'H')
  LODOP.SET_PRINT_COPIES(data.printQty)// 指定份数
  LODOP.SET_PRINT_PAGESIZE(1, width, height)
  LODOP.PRINT()
}
// 导出文件
export const exportExcel = function(url,params,fileName,callback){
  url(params).then(res => {
    if (res.type == 'application/json') {
      var reader = new FileReader()
      reader.addEventListener('loadend', function () {
        const obj = JSON.parse(reader.result)
        notification.error({
          message: '提示',
          description: obj.message
        })
      })
      reader.readAsText(res)
    } else {
       downloadExcel(res,fileName)
    }
    callback()
  })
}
// 下载excel
export const downloadExcel = function (data, fileName) {
  if (!data) { return }
  const a = moment().format('YYYYMMDDHHmmss')
  const fname = fileName + a
  const blob = new Blob([data], { type: 'application/vnd.ms-excel' })
  if (window.navigator && window.navigator.msSaveOrOpenBlob) {
    navigator.msSaveBlob(blob, fname + '.xlsx')
  } else {
    const link = document.createElement('a')
    link.style.display = 'none'
    var href = URL.createObjectURL(blob)
    link.href = href
    link.setAttribute('download', fname + '.xlsx')
    document.body.appendChild(link)
    link.click()
    document.body.removeChild(link)
    window.URL.revokeObjectURL(href) // 释放掉blob对象
  }
}

// pdf blob 转 base64
export const blobToBaseByPdf = function(data,callback){
  const reader = new FileReader()
  reader.readAsDataURL(new Blob([data], { type: 'application/pdf' }))
  reader.addEventListener('load', () => {
    callback(reader.result)
  })
}

// 打印html
export const jGPrint = function (data, type, callback, printLogParams) {
    if (!data) {
      return
    }
    const LODOP = getLodop(document.getElementById('LODOP_OB'), document.getElementById('LODOP_EM'))
    if (!LODOP) {
      confirm({
        title: '提示?',
        content: h => <div>打印控件未安装，请先下载并安装。安装完成后，刷新页面即可打印。</div>,
        okText: '立即下载',
        okType: 'danger',
        cancelText: '暂不打印',
        onOk () {
          var agent = navigator.userAgent.toLowerCase();
          if (agent.indexOf("win32") >= 0 || agent.indexOf("wow32") >= 0) {
              window.open('http://www.lodop.net/demolist/CLodop_Setup_for_Win32NT.zip')
          }else if (agent.indexOf("win64") >= 0 || agent.indexOf("wow64") >= 0) {
              window.open('http://www.lodop.net/download/CLodop_Setup_for_Win64NT_4.155EN.zip')
          }
        }
      })
      return
    }
    LODOP.SET_SHOW_MODE('HIDE_PAPER_BOARD', 1) //  隐藏底图上有模拟走纸板的条纹线
    //执行该语句之后，PRINT指令不再返回那个所谓“打印成功”，才能获取到打印状态
    LODOP.SET_PRINT_MODE("CATCH_PRINT_STATUS",true);
    //TaskID:任务id，Value：job代码
    LODOP.On_Return=function(TaskID,Value){
       console.log(TaskID,Value)
       // 已打印
       if(Value&&printLogParams){
         console.log('已打印,统计打印次数')
         printLog(printLogParams,callback)
       }else{
         callback()
       }
    };
    LODOP.ADD_PRINT_HTM(0, 0, '100%', '100%', data)
    LODOP.SET_PRINT_PAGESIZE(3,2100,100,"");
    if (type == 'preview') { //  预览
      LODOP.PREVIEW()
    } else if (type == 'print') { //  打印
      LODOP.PRINTA()
    }
}

// 打印
export const printFun = function(url,params,type,taskName,callback,printLogParams){
  url(params).then(res => {
    if (res.type == 'application/json') {
      var reader = new FileReader()
      reader.addEventListener('loadend', function () {
        const obj = JSON.parse(reader.result)
        notification.error({
          message: '提示',
          description: obj.message
        })
      })
      reader.readAsText(res)
    } else {
       console.log(res,'printFun')
       jGPrintPdf(res,type,taskName,printLogParams,callback)
    }
  })
}
// 获取系统信息
export const getSystemInfo = function (strINFOType,callback){
     LODOP=getLodop(); 
     if (LODOP.CVERSION) CLODOP.On_Return=function(TaskID,Value){callback(Value)}; 
     var strResult=LODOP.GET_SYSTEM_INFO(strINFOType);
     if (!LODOP.CVERSION) return strResult; else return "";
 }
// 打印记录保存
export const printLog = function(data,callback){
  getSystemInfo('NetworkAdapter.1.IPAddress',function(ret){
    data.printIp =  ret
    printLogSave(data).then(res => {
      callback(res)
    })
  })
}
// 打印pdf
export const jGPrintPdf = function (data, type, taskName, printLogParams,callback) {
    if (!data) {
      return
    }
    const LODOP = getLodop(document.getElementById('LODOP_OB'), document.getElementById('LODOP_EM'))
    if (!LODOP) {
      confirm({
        title: '提示?',
        content: h => <div>打印控件未安装，请先下载并安装。安装完成后，刷新页面即可打印。</div>,
        okText: '立即下载',
        okType: 'danger',
        cancelText: '暂不打印',
        onOk () {
          var agent = navigator.userAgent.toLowerCase();
          if (agent.indexOf("win32") >= 0 || agent.indexOf("wow32") >= 0) {
              window.open('http://www.lodop.net/demolist/CLodop_Setup_for_Win32NT.zip')
          }else if (agent.indexOf("win64") >= 0 || agent.indexOf("wow64") >= 0) {
              window.open('http://www.lodop.net/download/CLodop_Setup_for_Win64NT_4.155EN.zip')
          }
        }
      })
      return
    }
    blobToBaseByPdf(data, function (dataurl) {
      // console.log(dataurl)
      LODOP.PRINT_INIT(taskName)
      LODOP.SET_SHOW_MODE('HIDE_PAPER_BOARD', 1) //  隐藏底图上有模拟走纸板的条纹线
      //执行该语句之后，PRINT指令不再返回那个所谓“打印成功”，才能获取到打印状态
      LODOP.SET_PRINT_MODE("CATCH_PRINT_STATUS",true);
      //TaskID:任务id，Value：job代码
      LODOP.On_Return=function(TaskID,Value){
         console.log(TaskID,Value)
         // 已打印
         if(Value&&printLogParams){
           console.log('已打印,统计打印次数')
           printLog(printLogParams,callback)
         }else{
           callback()
         }
      };
      LODOP.ADD_PRINT_PDF(0, 0, '100%', '100%', dataurl.replace('data:application/pdf;base64,', ''))
      LODOP.SET_PRINT_PAGESIZE(3,2090,30,"");
      if (type == 'preview') { //  预览
        LODOP.PREVIEW()
      } else if (type == 'print') { //  打印
        LODOP.PRINTA()
      }
    })
}

// 获取打印机状态
export const getStatusValue = function (ValueType,ValueIndex,callback){
		const LODOP = getLodop(document.getElementById('LODOP_OB'), document.getElementById('LODOP_EM'))
		if (LODOP.CVERSION) {
      LODOP.On_Return=function(TaskID,Value){
        callback(Value)
      };
    }
		var strResult=LODOP.GET_VALUE(ValueType,ValueIndex);
		if (!LODOP.CVERSION) return callback(strResult); else return callback('');
	}