import Vue from 'vue'
import router from './router'
import store from './store'

import NProgress from 'nprogress' // progress bar
import '@/components/NProgress/nprogress.less' // progress bar custom style
import notification from 'ant-design-vue/es/notification'
import { setDocumentTitle, domTitle } from '@/utils/domUtil'
import { ACCESS_TOKEN } from '@/store/mutation-types'
import versionTood from '@/libs/versionUpdate'
NProgress.configure({ showSpinner: false }) // NProgress Configuration

const whiteList = ['login', 'register', 'registerResult'] // no redirect whitelist
const defaultRoutePath = '/home'

router.beforeEach((to, from, next) => {
  // 判断当前代码版本是否与服务器中代码版本一致，如不一致则刷新页面获取最新
  versionTood.isNewVersion()
  NProgress.start() // start progress bar
  to.meta && (typeof to.meta.title !== 'undefined' && setDocumentTitle(`${to.meta.title} - ${domTitle}`))
  let token = store.state.user.token || sessionStorage.getItem(ACCESS_TOKEN)
  token = token != 'null' && token != 'undefined' && token ? token : ''
  // console.log(token,'token')
  // 已登录
  if (token) {
    /* has token */
    if (to.path === '/user/login') {
      next({ path: defaultRoutePath })
      NProgress.done()
    } else {
      const userInfo = Vue.ls.get('userName')
      // console.log(userInfo)
      if (userInfo) {
        store.dispatch('SetInfo', userInfo)
      } else {
        store.dispatch('Logout').then(() => {
          // next({ path: '/user/login', query: { redirect: to.fullPath } })
          next({ path: '/user/login' })
        })
      }
      // console.log(store.getters.roles)
      if (store.getters.roles.permissionList.length === 0) {
        store
          .dispatch('GetInfo', token)
          .then(res => {
            const roles = res
            // console.log(roles, 'roles')
            if (roles.permissionList.length == 0 || !roles) {
              store.dispatch('Logout').then(() => {
                store.dispatch('ResetRoutes').then(() => {
                  // next({ path: '/user/login', query: { redirect: to.fullPath } })
                  next({ path: '/user/login' })
                })
              })
            } else {
              store.dispatch('GenerateRoutes', { roles }).then(() => {
                // 根据roles权限生成可访问的路由表
                next({ ...to, replace: true })
                // 动态添加可访问路由表
                // router.addRoutes(store.getters.addRouters)
                // const redirect = decodeURIComponent(from.query.redirect || to.path)
                // if (to.path === redirect) {
                //   // hack方法 确保addRoutes已完成 ,set the replace: true so the navigation will not leave a history record
                //   next({ ...to, replace: true })
                // } else {
                //   // 跳转到目的路由
                //   next({ path: redirect })
                // }
              })
            }
          })
          .catch(() => {
            notification.error({
              message: '错误',
              description: '请求用户信息失败，请重试'
            })
            store.dispatch('Logout').then(() => {
              // next({ path: '/user/login', query: { redirect: to.fullPath } })
              next({ path: '/user/login' })
            })
          })
      } else {
        next()
      }
    }
  } else {
    // 未登录
    if (whiteList.includes(to.name)) {
      // 在免登录白名单，直接进入
      next()
    } else {
      // next({ path: '/user/login', query: { redirect: to.fullPath } })
      next({ path: '/user/login' })
      NProgress.done() // if current page is login will not trigger afterEach hook, so manually handle it
    }
  }
})

router.afterEach(() => {
  NProgress.done() // finish progress bar
})
